%spherium_defaults
%
% S  Structure with parameters that the user will alter
% H  Data structure internal to spherium. i.e. user cannot change via
%    loading a file.
function [S,H] = spherium_defaults

%% VIEW SETTINGS %%

%Camera roll increment /degrees
S.camroll_increment_deg = 5;

%Pan toggle
S.toggle_pan = 0 ;

%Camera roll toggle
S.toggle_roll = 0 ;

%Camera roll about x axis toggle
S.toggle_rollx = 1;

%Camera roll about y axis toggle
S.toggle_rolly = 0;

%Camera roll about z axis toggle
S.toggle_rollz = 0;

%Camera settings
S = spherium_default_view(S);

%%

%% OUTPUT SETTINGS %%

%Output image size and resolution in dots per inch
S.output_size = 'A5 portrait';
H.output_sizes = {
    'A6 landscape',...
    'A6 portrait',...
    'A5 landscape',...
    'A5 portrait',...
    'A4 landscape',...
    'A4 portrait',...
    'A3 landscape',...
    'A3 portrait',...
    'A2 landscape',...
    'A2 portrait',...
    'A1 landscape',...
    'A1 portrait',...
    'A0 landscape',...
    'A0 portrait',...
    '6" x 4"',...
    '7" x 5"',...
    '8" x 6"',...
    '10" x 12"',...
    '10" x 15"'
    };
S.output_DPI = 300;

%Default output filename
S.filename = 'spherium image.png';

%%

%% SPHERIA SETTINGS %%

%Spheria and their default parameters
S.spheria = 'ammonite';
[S.spherias, S.SPHERIA] = spherium_get_spheria_defaults( 'spherium\SPHERIA' );

%Handle for surface plot
H.surface_handle = [];

%Handle for spiral plot figure
H.ammonite_spiral_plot = [];

%%

%% SURFACE & COLOUR %%

%Plot rendering. Note OpenGL is usually the best but may not be avaliable
%on all platforms
S.renderer = 'OpenGL';
H.renderers = {
    'OpenGL',...
    'zbuffer',...
    };

%Surface material
S.material = 'default';
H.materials = {
    'default',...
    'shiny',...
    'dull',...
    'metal'
    };

%Transparency?
S.transparency = 0;

%Holes (legacy option - not in GUI but could still be used. Implemented in spherium_update.m)
S.holes = 0;

%Texture map?
S.texture = 0;

%Add x,y,z axis to plot
S.add_axis = 0;

%Add colorbar
S.add_colorbar = 0;
H.colorbar_handle = [];

%Colour maps
H.colour_maps = {
    'hsv',...       %Hue-saturation-value color map.
    'hot',...       %Black-red-yellow-white color map.
    'gray',...      %Linear gray-scale color map.
    'bone',...      %Gray-scale with tinge of blue color map.
    'copper',...    %Linear copper-tone color map.
    'pink',...      %Pastel shades of pink color map.
    'white',...     %All white color map. Use with a non-white NaN colour
    'flag',...      %Alternating red, white, blue, and black color map.
    'lines',...     %Color map with the line colors.
    'colorcube',... %Enhanced color-cube color map.
    'vga',...       %Windows colormap for 16 colors.
    'jet',...       %Variant of HSV.
    'black & white',... %Alternating black and white - similar to 'prism'
    'prism',...     %Prism color map.
    'cool',...      %Shades of cyan and magenta color map.
    'autumn',...    %Shades of red and yellow color map.
    'spring',...    %Shades of magenta and yellow color map.
    'winter',...    %Shades of blue and green color map.
    'summer',...    %Shades of green and yellow color map.
    'bespoke'};

S.colour_map = 'jet';

%Colour functions
S.colorfunction = 'None';
H.colorfunctions = {
    'None',...
    'Log',...
    'Sine',...
    'Sine 10',...
    'Sine 200'
    };

%%

%% LIGHTING SETTINGS %%

%Lighting algorithm
S.lighting_model = 'phong';
H.lighting_models = {
    'phong',...
    'gouraud',...
    'flat',...
    'none'
    };

%Light styles
H.light_styles = {
    'infinite',...
    'local',...
    'none'
    };

%Light 3D rotation toggle flag
S.light_toggle = 0;

%List of lights
H.lights = {
    'Light 1',...
    'Light 2',...
    'Light 3'
    };
S.current_light = 1;

%Lighting color
for n=1:3
    S.light(n).light_color = [1 1 1];
end

%Light style
S.light(1).light_style = 'local';
S.light(2).light_style = 'none';
S.light(3).light_style = 'none';

%Light coordinates
for n=1:3
    S.light(n).range = 500;
    S.light(n).azi_deg = 0;
    S.light(n).elev_deg = 0;
end

%Light arrow
S.light_arrow = 0;

%%

%% MISC SETTINGS %%

%Default status box message
S.status = spherium_status('default');

%End of code