%spherium_gui
% GUI front end for spherium.m

%%%% MAIN GUI FUNCTIONS %%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function varargout = spherium_gui(varargin)

%% Initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @spherium_gui_OpeningFcn, ...
    'gui_OutputFcn',  @spherium_gui_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

%%

%% spherium opening function. This executes just before spherium is made visible.
function spherium_gui_OpeningFcn(hObject, eventdata, handles, varargin)

%Set up global variables for main window GUI handles h, fixed data store H,
%user modifiable data store S, spheria GUI handles s
global h H S s
h = handles;

%Ensure plot axis starts blank
axes( h.AXESspherium );
axis off;

%Center figure
figcentre( h.FIGUREspherium );

%Turn off warnings (e.g. 'divide by zero')
warning off

%Load spherium logo
axes( h.AXESlogo );
image( imread('spherium\spherium_logo.png' ));
axis tight
axis equal
axis off

% Load previous GUI settings, or default
[S,H] = spherium_defaults;
if exist('spherium\last_spherium_settings.mat','file')
    load('spherium\last_spherium_settings.mat');
end
H.colorbar_handle = [];
H.surface_handle = [];

%Load other data in spherium_store and store this in the handles array
load('spherium\spherium_store');
h.last_lookin_dir = last_lookin_dir;
if exist(last_lookin_dir,'dir')==0
    h.last_lookin_dir = pwd;
end

%Initialise mouse button status
h.button = 'not clicked';

%Set GUI flag that it is open and ready to accept windowbuttonmove
%callbacks
h.GUIisopen = 1;

%Update GUI from defaults
spherium_update_gui_from_S;

%Load current spheria parameter GUI
eval( S.spheria );
spheria_handle = findobj('tag',['FIGURE',S.spheria]);

%Align guis
gui_snap( h.FIGUREspherium, spheria_handle );

%Update spherium
spherium_update;
spherium_update_view_from_S;

%%

%% Plot type popup menu
function POPUPMENUspheria_Callback(hObject, eventdata, handles)
global h H S s

%Close previous spheria GUI
delete( findobj('tag',['FIGURE',S.spheria]) );

%Get string from menu
S.spheria = S.spherias{ get( h.POPUPMENUspheria, 'value' ) };

%Load new GUI
eval( S.spheria );
spheria_handle = findobj('tag',['FIGURE',S.spheria]);

%Align guis
gui_snap( h.FIGUREspherium,spheria_handle );

%Update GUI
spherium_update;
spherium_update_view_from_S;

%%

%% spherium output function. Outputs from this function are returned to the command line.
function varargout = spherium_gui_OutputFcn(hObject, eventdata, handles)

%%

%% Close function
function spherium_close(hObject, eventdata, handles)
global h H S s

%Saves settings (in structure S) to a .mat file
try
    save spherium\last_spherium_settings S
    last_lookin_dir = h.last_lookin_dir;
    save spherium\spherium_store last_lookin_dir
catch err
    %Possible error is one may not have write-access to the current drive
    errordlg( {'Have you got write access to the current drive?',...
        'If not spherium will not be able to save your settings on exiting.'},...
        'spherium error' );
end

%Close spherium GUI
delete( findobj('tag',['FIGURE',S.spheria] ));

%Closes the spherium main GUI
delete( h.FIGUREspherium );

%Remove current directory tree from MATLAB path
rmpath(genpath(pwd))

%%

%% Window Button Up function
function windowbuttonup(hObject, eventdata, handles)
global h H S s

%Set mouse button click status
h.button='not clicked';

%%

%% Window Button Down function
function windowbuttondown(hObject, eventdata, handles)
global h H S s

%Set flag in handles structure which defines the type of click event.
h.button = get(gcf, 'SelectionType');

%Double left mouse button click
if strcmp(h.button,'open')
    h.button = 'double click';
    
    %Single left mouse click
elseif strcmp(h.button,'normal')
    h.button = 'single click';
    
    %Shift + click or right & left mouse click
elseif strcmp(h.button,'extend')
    h.button = 'shift & click';
    
    %Control + click or right mouse click
elseif strcmp(h.button,'alt')
    h.button = 'ctrl & click';
else
    h.button='not clicked';
end

%Store 'click point' coordinates (pixels from bottom left of the figure)
cp=get(h.FIGUREspherium,'currentpoint');
h.click_point_x = cp(1,1);
h.click_point_y = cp(1,2);

%Check 'click point' coordinates are within axes
ax_pos = get( h.AXESspherium, 'position' );
xv = [ ax_pos(1), ax_pos(1)+ax_pos(3), ax_pos(1)+ax_pos(3), ax_pos(1) ];
yv = [ ax_pos(2), ax_pos(2), ax_pos(2) + ax_pos(4), ax_pos(2) + ax_pos(4) ];
h.within_axes = inpolygon( h.click_point_x, h.click_point_y, xv, yv );

%Get light azimuth and elevation when mouse was depressed
p = get( H.light_handle(S.current_light),'position' );
[h.light_azi_rad, h.light_elev_rad ,range] = cart2sph( p(1), p(2), p(3) );

%%

%% Window Button Motion function
function windowbuttonmotion(hObject, eventdata, handles)
global h H S s

if isfield( h,'GUIisopen');
    if (gcf==h.FIGUREspherium) && ( ~strcmp( h.button,'not clicked') ) && ( h.within_axes == 1 )
        
        %Store 'click point' coordinates (pixels from bottom left of the figure)
        cp=get(h.FIGUREspherium,'currentpoint');
        x = cp(1,1);
        y = cp(1,2);
        
        %Get size of figure in pixels
        p = get( h.FIGUREspherium,'position' );
        
        %Determine azimuth and elevation change from mouse movement
        u = 2 * (x - h.click_point_x )/p(3) ;
        if u < -1
            u = -1;
        elseif u > 1
            u = 1;
        end
        az = 2*asin(u);
        v = 2 * (y - h.click_point_y )/p(4) ;
        if v < -1
            v = -1;
        elseif u > 1
            v = 1;
        end
        el = 2*asin(v);
        
        %Only modify light if light toggle button is on
        if S.light_toggle == 1
            
            %Modify light source angle depending of location of mouse button
            %within axes
            az = az + h.light_azi_rad;
            el = el + h.light_elev_rad;
            
            %Wrap back into a sensible range of angles (radians)
            [x,y,z] = sph2cart( az, el, S.light( S.current_light ).range );
            [az,el,r] = cart2sph( x,y,z );
            
            %Update light azimuth and elevation
            S.light( S.current_light ).azi_deg = (180/pi)*az;
            S.light( S.current_light ).elev_deg = (180/pi)*el;
            set( h.EDITlightazi,'string',num2str( S.light( S.current_light ).azi_deg ) );
            set( h.EDITlightelev,'string',num2str( S.light( S.current_light ).elev_deg ) );
            
            %Modify lighting
            set(H.light_handle(S.current_light),...
                'position', [x,y,z] );
            
        else
            %Otherwise rotate the axes via changing the camera viewpoint
            if get( h.TOGGLEpan,'value')==0 && get( h.TOGGLEroll,'value')==0
                
                %Orbit the camera (default)
                if strcmp(get(gcf, 'SelectionType'),'normal')
                    camorbit( -5*u, -5*v, 'camera' );
                else
                    %Holding down extra buttons rotates about the data
                    camorbit( -5*u, -5*v, 'data' );
                end
            elseif get( h.TOGGLEpan,'value')==1
                
                %Pan the camera
                campan( -u/5, -v/5, 'camera' );
                
            elseif get( h.TOGGLEroll,'value')==1
                
                %Roll the camera
                if v*u>0
                    camroll( (3*v+u) );
                else
                    camroll( (3*v+u) );
                end
            end
            
            %Update GUI following change of view
            spherium_update_S_from_view;
            spherium_update_gui_from_S;
            
            %Flush graphic buffer
            drawnow;
        end
    end
end

%%

%%%% INPUTS %%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Load default settings pushbutton
function PUSHdefault_Callback(hObject, eventdata, handles)
global h H S s

%Close previous spheria GUI
delete( findobj('tag',['FIGURE',S.spheria]) );

%Delete any previous surfaces or colorbars
delete( H.colorbar_handle ); H.colorbar_handle = [];
delete( H.surface_handle ); H.surface_handle = [];
cla;

%Copy default holes, texture and transparency maps
copyfile( 'spherium\default_texture.jpg','spherium\texture.jpg','f');
copyfile( 'spherium\default_holes.jpg','spherium\holes.jpg','f');
copyfile( 'spherium\default_transparency.jpg','spherium\transparency.jpg','f');

%Get spherium default settings
[S,H] = spherium_defaults;

%Load new GUI
eval( S.spheria );

%Align guis
gui_snap( h.FIGUREspherium,findobj('tag',['FIGURE',S.spheria]) );

%Update GUI
spherium_update_gui_from_S;
spherium_update;
spherium_update_view_from_S;

%%

%% Load settings pushbutton
function PUSHloadsettings_Callback(hObject, eventdata, handles)
global h H S s

%Return path to valid .mat file obtained via Explorer navigation by user
%which contains spherium setup information.
[filename, pathname, filterindex] = uigetfile([h.last_lookin_dir,'\*.mat'],...
    'Choose a .mat file containing spherium settings');

%Try and load settings in .mat file
if pathname ~=0
    h.last_lookin_dir = pathname;
    
    %Delete any previous surfaces or colorbars
    axes( h.AXESspherium );
    delete( H.colorbar_handle ); H.colorbar_handle = [];
    delete( H.surface_handle ); H.surface_handle = [];
    cla;
    
    %Store current user settings
    old_S = S;
    try
        %Load .mat file and update handles array of GUI with information contained
        %within the .mat file. If this doesn't work flag up an error dialogue box
        load( [pathname, '\', filename] );
        
        %Update GUI
        spherium_update_gui_from_S;
        spherium_update;
        spherium_update_view_from_S;
    catch
        errordlg('.mat file selected does not contain the correct inputs for spherium','spherium error!')
        
        %Update GUI with original settings
        S = old_S;
        
        %Update GUI
        spherium_update_gui_from_S;
        spherium_update;
        spherium_update_view_from_S;
        return
    end
end

%%

% Load transparency .jpg file
function PUSHloadtransparency_Callback(hObject, eventdata, handles)
global h H S s
[filename, pathname, filterindex] = uigetfile([h.last_lookin_dir,'\*.jpg'],...
    'Choose a .jpg file comprising the transparency map');
if pathname ~=0
    h.last_lookin_dir = pathname;
    copyfile( [pathname,'\',filename],'spherium\transparency.jpg','f');
    spherium_update;
end

%%

%% Load texture .jpg file %%
function PUSHloadtexture_Callback(hObject, eventdata, handles)
global h H S s
[filename, pathname, filterindex] = uigetfile([h.last_lookin_dir,'\*.jpg'],...
    'Choose a .jpg file comprising the texture map');
if pathname ~=0
    h.last_lookin_dir = pathname;
    copyfile( [pathname,'\',filename],'spherium\texture.jpg','f');
    spherium_update;
end

%%

%%%% OUTPUTS %%%%

%% Save .PNG file of current spherium pushbutton
function PUSHsavepng_Callback(hObject, eventdata, handles)
global h H S s

%Start clock
t0=cputime;

%Update status
S.status = spherium_status('Saving PNG');
set( h.TEXTstatus,'string',S.status );

%Save PNG file
savepng;

%Update status
S.status = ['PNG image saved in ',num2str(cputime-t0),' s.  ',...
    spherium_status('default') ];
set( h.TEXTstatus,'string',S.status );

%%

%% Save .stl (stereo lithography file for 3D printing) file of current spheria
function PUSHsavestl_Callback(hObject, eventdata, handles)
global h H S s

%Start clock
t0=cputime;

%Update status
S.status = spherium_status('Saving .stl');
set( h.TEXTstatus,'string',S.status );

%Save .stl file

%If transparency applied, drill holes in the surface
if S.transparency==1;
    
    %Set max value of transparency for holes to be 'drilled'
    Tmax = 0.1;
    
    %Load image of texture from file texture.jpg
    [T,map] = imread( 'spherium\transparency.jpg' );
    
    %Get size of surface matrix
    C = get( H.surface_handle, 'CData');
    [M,N,P] = size(C);
    
    %Interpolate image
    T = interp_image( T , N , M , '' ) ;
    
    %Make sure it is only one 'page' i.e. greyscale. If not mean over the
    %other colours.
    [r,c,p] = size(T);
    if p~=1
        TT = zeros(r,c);
        TT = (1/3) * ( T(:,:,1)+T(:,:,2)+T(:,:,3) );
        T = TT;
    end
    clear II
    
    %Convert into range 0-1 and flip
    T=T-min(min(T));
    T = T/max(max(T));
    T = flipdim(T,1);
    
    %Store original x,y,z coordinates
    x = get(H.surface_handle,'Xdata');
    y = get(H.surface_handle,'Ydata');
    z = get(H.surface_handle,'Zdata');
    xx = get(H.surface_handle,'Xdata');
    yy = get(H.surface_handle,'Ydata');
    zz = get(H.surface_handle,'Zdata');
    
    %Drill holes via setting x,y,z coordinates to be NaN
    x(T<Tmax) = NaN;
    y(T<Tmax) = NaN;
    z(T<Tmax) = NaN;
    clear T
    
    %Update surface
    set(H.surface_handle,'Xdata',x);
    set(H.surface_handle,'Ydata',y);
    set(H.surface_handle,'Zdata',z);
    clear x y z
end

%Export .stl file of current surface
stlwrite( ['spherium image ',strrep(datestr(now),':','-'),'.stl'],...
    surf2patch(H.surface_handle,'triangles'));

%Restore figure, if transparency has caused holes to be drilled
if S.transparency==1;
    set(H.surface_handle,'Xdata',xx);
    set(H.surface_handle,'Ydata',yy);
    set(H.surface_handle,'Zdata',zz);
    clear xx yy zz
end

%Update status
S.status = ['.stl file saved in ',num2str(cputime-t0),' s.  ',...
    spherium_status('default') ];
set( h.TEXTstatus,'string',S.status );


%% Save .PNG file of current spherium
function filename = savepng
global h H S s

%Obtain colormap and colour limits of spherium axes
axes(h.AXESspherium);
cmap = colormap;
cax = caxis;

%Create new figure containing a copy of the spherium axes.
new_fig = figure('color',[1 1 1],'name','spherium png output','renderer',S.renderer);
[ new_fig, print_DPI ] = fig4print( new_fig, S.output_size,S.output_DPI );

%Determine figure width in cm
set( new_fig, 'units','centimeters' );
p = get( new_fig, 'position' );
fig_width_cm = p(3);

%Copy contents of GUI axis into new figure
new_ax = copyobj(h.AXESspherium,new_fig);
colormap(cmap);
caxis(cax);

%Fit new_ax to figure
fitax2fig( new_fig, new_ax )

%Modify print DPI since figure window has been reduced, but fitax2fig
%should not have modified the figure aspect ratio!
set( new_fig, 'units','centimeters' );
p = get( new_fig, 'position' );
print_DPI = print_DPI * fig_width_cm / p(3);

%Add colorbar if option selected
if S.add_colorbar==1
    set( new_fig, 'units','normalized' );
    colorbar('fontsize',8,...
        'location','North',...
        'position',[0.02 0.95 0.4 0.03]);
end

%Print figure to a PNG file then close figure
filename = [S.spheria,' ',strrep( datestr(now), ':','-' ),'.png'];
print( new_fig, filename, '-dpng', ['-r',num2str(print_DPI)] );
close(new_fig);

%Crop image
croppic( filename, [255 255 255] );

%Restore GUI with axis, colorbars etc. Also force light toggle to be off.
axes(h.AXESspherium);
S.light_toggle = 0;

%%

%% Print 360 views to indexed PNG files in a directory
function PUSH360_Callback(hObject, eventdata, handles)
global h H S s

%Start clock
t0 = tic;

%Work out number of camera rolls in 360
N = floor( 360 / S.camroll_increment_deg );

%Display message to status box
S.status = ['Creating first of ',num2str(N+1),' images of a 360deg roll ...'];
set( h.TEXTstatus,'string', S.status );

%Create a new directory
dirname = [S.spheria,' ',strrep( datestr(now), ':','-' ) ];
mkdir( dirname );

%Save PNG file of current view
filename = savepng;
movefile( filename, [dirname, '\000 ', filename] );

%Roll camera and save indexed PNG files
for n=1:N
    
    %Update status box
    S.status = ['Creating image ',num2str(n+1),...
        ' of ',num2str(N+1),' images of a 360deg roll ...'];
    set( h.TEXTstatus, 'string', S.status );
    
    %Orbit camera
    if S.toggle_rollx == 1
        camroll( S.camroll_increment_deg );
    elseif S.toggle_rolly == 1
        camorbit( 0, S.camroll_increment_deg );
    elseif S.toggle_rollz == 1
        camorbit( S.camroll_increment_deg, 0 );
    end
    
    %Modify light position vector to be that of the camera
    p = campos;
    set(H.light_handle(S.current_light),'position',p);
    [az,el,S.light( S.current_light ).range] = cart2sph( p(1), p(2), p(3) );
    S.light( S.current_light ).elev_deg = el*180/pi;
    S.light( S.current_light ).azi_deg = az*180/pi;
    set( h.EDITlightrange,'string',num2str( S.light( S.current_light ).range) );
    set( h.EDITlightazi,'string',num2str( S.light( S.current_light ).azi_deg) );
    set( h.EDITlightelev,'string',num2str( S.light( S.current_light ).elev_deg) );
    
    %Update GUI
    spherium_update_S_from_view;
    spherium_update_gui_from_S;
    
    %Save PNG file
    filename = savepng;
    movefile( filename, [ dirname, '\',leadingzero(n,3),filename ] );
end
set( h.TEXTstatus,'string', ['360deg roll of images done in ',num2str( toc(t0)/60 ),' minutes.'] );

%%

%% Output image size popup menu
function POPUPMENUoutputsize_Callback(hObject, eventdata, handles)
global h H S s
S.output_size = H.output_sizes{ get( h.POPUPMENUoutputsize, 'value' ) };

%%

%% Output image Dots Per Inch edit box
function EDITDPI_Callback(hObject, eventdata, handles)
global h H S s

%Get DPI
S.output_DPI = str2num( get(h.EDITDPI,'string') );
S.output_DPI = S.output_DPI(1);

%%

%% Save settings pushbutton
function PUSHsavesettings_Callback(hObject, eventdata, handles)
global h H S s

%Return path to valid .mat file obtained via Explorer navigation by user
%which will contains spherium setup information.
[filename, pathname, filterindex] = uiputfile([h.last_lookin_dir,'\*.mat'],...
    'Save spherium GUI settings in a .mat file');

%Save .mat file containing GUI settings
if ~isequal(filename,0) && ~isequal(pathname,0)
    save( [pathname,'\',filename], 'S' );
end

%%

%%%% SURFACE AND COLOUR %%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Apply transparency map
function CHECKtransparency_Callback(hObject, eventdata, handles)
global h H S s

%Get value of checkbox
S.transparency = get(h.CHECKtransparency,'value');

%Update GUI
spherium_update;

%%

%Apply texture map
function CHECKtexture_Callback(hObject, eventdata, handles)
global h H S s

%Get value of checkbox
S.texture = get(h.CHECKtexture,'value');

%Update GUI
spherium_update;

%%

%% Colour map popupmenu
function POPUPMENUcolormap_Callback(hObject, eventdata, handles)
global h H S s

%Get string from menu
S.colour_map = H.colour_maps{ get( h.POPUPMENUcolormap, 'value' ) };

%Load bespoke colormap if this option is set
if strcmp(S.colour_map,'bespoke')==1
    [filename, pathname, filterindex] = uigetfile('*.mat', 'Select bespoke colormap file');
    if filename == 0
        load( 'bespoke_colormap_default.mat');
    else
        load([pathname,'\',filename]);
    end
    
    %Set colormap
    colormap(map);
end

%Update GUI
spherium_update;

%%

%% Colour function
function POPUPMENUcolorfunction_Callback(hObject, eventdata, handles)
global h H S s

%Get string from menu
S.colorfunction = ...
    H.colorfunctions{ get(h.POPUPMENUcolorfunction,'value') };

%Update GUI
spherium_update;

%%

%% Add axis checkbox
function CHECKaxis_Callback(hObject, eventdata, handles)
global h H S s

%Get value of checkbox
S.add_axis = get(h.CHECKaxis,'value');

%Update GUI
spherium_update;

%%

%% Add colorbar checkbox
function CHECKcolorbar_Callback(hObject, eventdata, handles)
global h H S s

%Get value of checkbox
S.add_colorbar = get(h.CHECKcolorbar,'value');

%Update GUI
spherium_update;

%%

%%%% LIGHTING %%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Select light popupmenu
function POPUPMENUselectlight_Callback(hObject, eventdata, handles)
global h H S s

%Modify current light selected
S.current_light = get(h.POPUPMENUselectlight,'value' );

%Update GUI
spherium_update_gui_from_S

%%

%% Light range coordinate edit box
function EDITlightrange_Callback(hObject, eventdata, handles)
global h H S s

%Update handles array based upon new light position coordinates
S.light( S.current_light ).range = ...
    str2num( get( h.EDITlightrange,'string' ) );

%Modify lighting
[x,y,z] = sph2cart( (pi/180)*S.light( S.current_light ).azi_deg,...
    (pi/180)*S.light( S.current_light ).elev_deg,...
    S.light( S.current_light ).range );
set( H.light_handle(S.current_light),...
    'position', [x,y,z] );

%%

%% Light azimuth coordinate edit box
function EDITlightazi_Callback(hObject, eventdata, handles)
global h H S s

%Update handles array based upon new light position coordinates
S.light( S.current_light ).azi_deg = ...
    str2num( get( h.EDITlightazi,'string' ) );

%Modify lighting
[x,y,z] = sph2cart( (pi/180)*S.light( S.current_light ).azi_deg,...
    (pi/180)*S.light( S.current_light ).elev_deg,...
    S.light( S.current_light ).range );
set( H.light_handle(S.current_light),...
    'position', [x,y,z] );


%%

%% Light elevation coordinate edit box
function EDITlightelev_Callback(hObject, eventdata, handles)
global h H S s

%Update handles array based upon new light position coordinates
S.light( S.current_light ).elev_deg = ...
    str2num( get( h.EDITlightelev,'string' ) );

%Modify lighting
[x,y,z] = sph2cart( (pi/180)*S.light( S.current_light ).azi_deg,...
    (pi/180)*S.light( S.current_light ).elev_deg,...
    S.light( S.current_light ).range );
set( H.light_handle(S.current_light),...
    'position', [x,y,z] );


%%

%% Light direction arrow toggle
function CHECKlightarrow_Callback(hObject, eventdata, handles)
global h H S s

%TBD

%%

%% Camlight pushbutton
function PUSHcamlight_Callback(hObject, eventdata, handles)
global h H S s

%Modify light position vector to be that of the camera
p = campos;
set(H.light_handle(S.current_light),'position',p);
[az,el,S.light( S.current_light ).range] = cart2sph( p(1), p(2), p(3) );
S.light( S.current_light ).elev_deg = el*180/pi;
S.light( S.current_light ).azi_deg = az*180/pi;
set( h.EDITlightrange,'string',num2str( S.light( S.current_light ).range) );
set( h.EDITlightazi,'string',num2str( S.light( S.current_light ).azi_deg) );
set( h.EDITlightelev,'string',num2str( S.light( S.current_light ).elev_deg) );

%%

%% Light color pushbutton
function PUSHlightcolor_Callback(hObject, eventdata, handles)
global h H S s

%Determine light color from color picker
S.light(S.current_light).light_color =...
    uisetcolor( S.light(S.current_light).light_color, 'Set light colour' );
set( h.PUSHlightcolor,'backgroundcolor',S.light(S.current_light).light_color );
set( H.light_handle(S.current_light),...
    'color',S.light(S.current_light).light_color );

%%

%% Lighting style popupmenu
function POPUPMENUlightingstyle_Callback(hObject, eventdata, handles)
global h H S s

%Get lighting style type from menu and update properties of Light # v
v = get( h.POPUPMENUlightingstyle, 'value' );
S.light(S.current_light).light_style = H.light_styles{v};
if strcmp( H.light_styles{v}, 'none' )
    set( H.light_handle(S.current_light),'visible','off' );
else
    set( H.light_handle(S.current_light),'style',...
        S.light(S.current_light).light_style,'visible','on' );
end

%%

%% Lighting style popupmenu
function POPUPMENUlightingmodel_Callback(hObject, eventdata, handles)
global h H S s

%Get lighting style type from menu and update properties of Light # v
v = get( h.POPUPMENUlightingstyle, 'value' );
S.light(S.current_light).light_style = H.light_styles{v};
if strcmp( H.light_styles{v}, 'none' )
    set( H.light_handle(S.current_light),'visible','off' );
else
    set( H.light_handle(S.current_light),'style',...
        S.light(S.current_light).light_style,'visible','on' );
end

%%

%% 3D light mode toggle button.
function TOGGLElight_Callback(hObject, eventdata, handles)
global h H S s

%Update light toggle status
S.light_toggle = get( h.TOGGLElight,'value');

%Modify 3D rotation mode depending on toggle
axes( h.AXESspherium );
if S.light_toggle == 1
    S.status = spherium_status('light toggled');
else
    S.status = spherium_status('default');
end
set( h.TEXTstatus,'string',S.status );

%%

%%%% CAMERA VIEW - CONTROLS RIGHT OF AXIS %%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Zoom in pushbutton +
function PUSHzoomin_Callback(hObject, eventdata, handles)
global h H S s
axes(h.AXESspherium);
camzoom(1.2);
spherium_update_S_from_view;
spherium_update_gui_from_S;

%%

%% Zoom out pushbutton -
function PUSHzoomout_Callback(hObject, eventdata, handles)
global h H S s
axes(h.AXESspherium);
camzoom(1/1.2);
spherium_update_S_from_view;
spherium_update_gui_from_S;

%%

%% Pan up pushbutton
function PUSHup_Callback(hObject, eventdata, handles)
global h H S s
axes(h.AXESspherium);
camdolly(0,-0.02,0);
spherium_update_S_from_view;
spherium_update_gui_from_S;

%%

%% Pan down pushbutton
function PUSHdown_Callback(hObject, eventdata, handles)
global h H S s
axes(h.AXESspherium);
camdolly(0,0.02,0);
spherium_update_S_from_view;
spherium_update_gui_from_S;

%%

%% Pan right pushbutton
function PUSHright_Callback(hObject, eventdata, handles)
global h H S s
axes(h.AXESspherium);
camdolly(-0.02,0,0);
spherium_update_S_from_view;
spherium_update_gui_from_S;

%%

%% Pan left pushbutton
function PUSHleft_Callback(hObject, eventdata, handles)
global h H S s
axes(h.AXESspherium);
camdolly(0.02,0,0);
spherium_update_S_from_view;
spherium_update_gui_from_S;

%%

%% Toggle camera pan %%
function TOGGLEpan_Callback(hObject, eventdata, handles)
global h H S s

%Get toggle value from GUI
S.toggle_pan = get( h.TOGGLEpan, 'value' );

%Make sure camera roll is untoggled
if S.toggle_pan==1
    set( h.TOGGLEroll, 'value', 0 );
    S.toggle_roll = 0;
end

%%

%% Toggle camera roll about a specified axis %%
function TOGGLEroll_Callback(hObject, eventdata, handles)
global h H S s

%Get toggle value from GUI
S.toggle_roll = get( h.TOGGLEroll, 'value' );

%Make sure camera pan is untoggled
if S.toggle_roll==1
    set( h.TOGGLEpan, 'value', 0 );
    S.toggle_pan = 0;
end

%%

%% Toggle camera roll about axis x (out of screen)  %%
function TOGGLErollx_Callback(hObject, eventdata, handles)
global h H S s

%Get toggle value from GUI
S.toggle_rollx = get( h.TOGGLErollx, 'value' );

%Make sure y, z toggles are set to zero
if S.toggle_rollx==1
    set( h.TOGGLErolly, 'value', 0 );
    set( h.TOGGLErollz, 'value', 0 );
    S.toggle_rolly = 0;
    S.toggle_rollz = 0;
end

%%

%% Toggle camera roll about y axis (vertical to screen) %%
function TOGGLErolly_Callback(hObject, eventdata, handles)
global h H S s

%Get toggle value from GUI
S.toggle_rolly = get( h.TOGGLErolly, 'value' );

%Make sure x, z toggles are set to zero
if S.toggle_rolly==1
    set( h.TOGGLErollx, 'value', 0 );
    set( h.TOGGLErollz, 'value', 0 );
    S.toggle_rollx = 0;
    S.toggle_rollz = 0;
end

%%

%% Toggle camera roll about z axis (horizontal to screen) %%
function TOGGLErollz_Callback(hObject, eventdata, handles)
global h H S s

%Get toggle value from GUI
S.toggle_rollz = get( h.TOGGLErollz, 'value' );

%Make sure x, y toggles are set to zero
if S.toggle_rollz==1
    set( h.TOGGLErollx, 'value', 0 );
    set( h.TOGGLErolly, 'value', 0 );
    S.toggle_rollx = 0;
    S.toggle_rolly = 0;
end

%%

%% Set angle increment or camera orbits
function EDITcamrollinc_Callback(hObject, eventdata, handles)
global h H S s
S.camroll_increment_deg = str2num( get( h.EDITcamrollinc,'string' ));

%%

%% Roll clockwise by increment
function PUSHrollclockwise_Callback(hObject, eventdata, handles)
global h H S s
if S.toggle_rollx == 1
    camroll( S.camroll_increment_deg );
elseif S.toggle_rolly == 1
    camorbit( 0, S.camroll_increment_deg );
elseif S.toggle_rollz == 1
    camorbit( S.camroll_increment_deg, 0 );
end
spherium_update_S_from_view;
spherium_update_gui_from_S;

%%

%% Roll anticlockwise by increment
function PUSHrollanticlockwise_Callback(hObject, eventdata, handles)
global h H S s
if S.toggle_rollx == 1
    camroll( -S.camroll_increment_deg );
elseif S.toggle_rolly == 1
    camorbit( 0, -S.camroll_increment_deg );
elseif S.toggle_rollz == 1
    camorbit( -S.camroll_increment_deg, 0 );
end
spherium_update_S_from_view;
spherium_update_gui_from_S;

%%

%%%% CAMERA VIEW %%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Default camera view %%
function PUSHcamdefault_Callback(hObject, eventdata, handles)
global h H S s
S = spherium_default_view(S);
spherium_update_view_from_S;

%Camera position
set( h.EDITcamposx,'string',num2str( S.campos_x ) );
set( h.EDITcamposy,'string',num2str( S.campos_y ) );
set( h.EDITcamposz,'string',num2str( S.campos_z ) );

%Camera target
set( h.EDITcamtargetx,'string',num2str( S.camtarget_x ) );
set( h.EDITcamtargety,'string',num2str( S.camtarget_y ) );
set( h.EDITcamtargetz,'string',num2str( S.camtarget_z ) );

%Camera up vector
set( h.EDITcamupx,'string',num2str( S.camup_x ) );
set( h.EDITcamupy,'string',num2str( S.camup_y ) );
set( h.EDITcamupz,'string',num2str( S.camup_z ) );

%Camera view angle
set( h.EDITcamviewangle,'string',num2str( S.camviewangle_deg ) );

%%

%% Camera position x
function EDITcamposx_Callback(hObject, eventdata, handles)
global h H S s
S.campos_x = str2num( get(h.EDITcamposx,'string') );
p = get( h.AXESspherium,'CameraPosition' );
set( h.AXESspherium,'CameraPosition',[S.campos_x,p(2),p(3)] );

%%

%% Camera position y
function EDITcamposy_Callback(hObject, eventdata, handles)
global h H S s
S.campos_x = str2num( get(h.EDITcamposx,'string') );
p = get( h.AXESspherium,'CameraPosition' );
set( h.AXESspherium,'CameraPosition',[S.campos_x,p(2),p(3)] );

%%

%% Camera position z
function EDITcamposz_Callback(hObject, eventdata, handles)
global h H S s
S.campos_x = str2num( get(h.EDITcamposx,'string') );
p = get( h.AXESspherium,'CameraPosition' );
set( h.AXESspherium,'CameraPosition',[S.campos_x,p(2),p(3)] );

%%

%% Camera target x
function EDITcamtargetx_Callback(hObject, eventdata, handles)
global h H S s
S.camtarget_x = str2num( get(h.EDITcamtargetx,'string') );
p = get( h.AXESspherium,'CameraTarget' );
set( h.AXESspherium,'CameraTarget',[S.camtarget_x,p(2),p(3)] );

%%

%% Camera target y
function EDITcamtargety_Callback(hObject, eventdata, handles)
global h H S s
S.camtarget_y = str2num( get(h.EDITcamtargety,'string') );
p = get( h.AXESspherium,'CameraTarget' );
set( h.AXESspherium,'CameraTarget',[p(1),S.camtarget_y,p(3)] );

%%

%% Camera target z
function EDITcamtargetz_Callback(hObject, eventdata, handles)
global h H S s
S.camtarget_z = str2num( get(h.EDITcamtargetz,'string') );
p = get( h.AXESspherium,'CameraTarget' );
set( h.AXESspherium,'CameraTarget',[p(1),p(2),S.camtarget_z] );

%%

%% Camera up vector x
function EDITcamupx_Callback(hObject, eventdata, handles)
global h H S s
S.camup_x = str2num( get(h.EDITcamupx,'string') );
p = get( h.AXESspherium,'CameraUpVector' );
set( h.AXESspherium,'CameraUpVector',[ S.camup_x, p(2),p(3) ] );

%%

%% Camera up vector  y
function EDITcamupy_Callback(hObject, eventdata, handles)
global h H S s
S.camup_y = str2num( get(h.EDITcamupy,'string') );
p = get( h.AXESspherium,'CameraUpVector' );
set( h.AXESspherium,'CameraUpVector',[ p(1),S.camup_y, p(3) ] );

%%

%% Camera up vector  z
function EDITcamupz_Callback(hObject, eventdata, handles)
global h H S s
S.camup_z = str2num( get(h.EDITcamupz,'string') );
p = get( h.AXESspherium,'CameraUpVector' );
set( h.AXESspherium,'CameraUpVector',[ p(1),p(2),S.camup_z ] );

%%

%% Camera view angle /degrees
function EDITcamviewangle_Callback(hObject, eventdata, handles)
global h H S s
S.camviewangle_deg = str2num( get(h.EDITcamviewangle,'string') );
set( h.AXESspherium,'CameraViewAngle',S.camviewangle_deg );

%%

%End of code






