%spherium_update_gui_from_S
% Updates the spherium GUI based upon settings stored within structure S
% held within the GUI data structure.

function spherium_update_gui_from_S
global h H S

%% Outputs %%

%Output image size and resolution in dots per inch
set( h.POPUPMENUoutputsize,'string',H.output_sizes );
set( h.POPUPMENUoutputsize,'value',...
    find( strcmp( S.output_size, H.output_sizes )==1 ));
set( h.EDITDPI,'string',num2str(S.output_DPI) );


%%

%% Other GUI controls %%

%Spheria type
set( h.POPUPMENUspheria, 'string', S.spherias );
set( h.POPUPMENUspheria,'value',...
    find( strcmp( S.spheria, S.spherias ) == 1 ) );

%Update status box
set( h.TEXTstatus,'string',S.status );

%Update version
set( h.TEXTversion,'string',spherium_version );

%%

%% View settings %%

%Camera position
set( h.EDITcamposx,'string',num2str( S.campos_x ) );
set( h.EDITcamposy,'string',num2str( S.campos_y ) );
set( h.EDITcamposz,'string',num2str( S.campos_z ) );

%Camera target
set( h.EDITcamtargetx,'string',num2str( S.camtarget_x ) );
set( h.EDITcamtargety,'string',num2str( S.camtarget_y ) );
set( h.EDITcamtargetz,'string',num2str( S.camtarget_z ) );

%Camera up vector
set( h.EDITcamupx,'string',num2str( S.camup_x ) );
set( h.EDITcamupy,'string',num2str( S.camup_y ) );
set( h.EDITcamupz,'string',num2str( S.camup_z ) );

%Camera view angle
set( h.EDITcamviewangle,'string',num2str( S.camviewangle_deg ) );

%Camera orbit increment
set( h.EDITcamrollinc,'string',num2str( S.camroll_increment_deg ) );

%Camera pan toggle
set( h.TOGGLEpan,'value',S.toggle_pan );

%Camera roll toggle
set( h.TOGGLEroll,'value',S.toggle_roll );

%Camera roll about x axis toggle
set( h.TOGGLErollx,'value',S.toggle_rollx );

%Camera roll about y axis toggle
set( h.TOGGLErolly,'value',S.toggle_rolly );

%Camera roll about z axis toggle
set( h.TOGGLErollz,'value',S.toggle_rollz );

%%

%% Surface & Colour settings %%

%Transparency
set( h.CHECKtransparency,'value',S.transparency );

%Add x,y,z axis to plot
set( h.CHECKaxis,'value',S.add_axis );

%Add colorbar
set( h.CHECKcolorbar,'value',S.add_colorbar );

%General colouring settings ...
set( h.POPUPMENUcolormap,'string',H.colour_maps );
set( h.POPUPMENUcolormap,'value',...
    find( strcmp( S.colour_map , H.colour_maps ) == 1 ) );
set( h.POPUPMENUcolorfunction,'string',H.colorfunctions );
set( h.POPUPMENUcolorfunction,'value',...
    find( strcmp( S.colorfunction , H.colorfunctions ) == 1 ) );

%Figure renderer
set( h.FIGUREspherium, 'Renderer', S.renderer );

%%

%% Lighting settings .... %%

%List of lights
set (h.POPUPMENUselectlight,'string',H.lights );
set( h.POPUPMENUselectlight,'value',S.current_light);

%Lightning model
set( h.POPUPMENUlightingmodel, 'string', H.lighting_models );
set( h.POPUPMENUlightingmodel,'value',...
    find( strcmp( S.lighting_model, H.lighting_models )) == 1 );

%Lighting style
set( h.POPUPMENUlightingstyle, 'string', H.light_styles );
set( h.POPUPMENUlightingstyle,'value',...
    find( strcmp( S.light(S.current_light).light_style, H.light_styles ) == 1 ));

%Color of light color pushbutton
set( h.PUSHlightcolor,'backgroundcolor',S.light(S.current_light).light_color );

%Light position edit boxes
set( h.EDITlightrange,'string',num2str( S.light(S.current_light).range ) );
set( h.EDITlightazi,'string',num2str( S.light(S.current_light).azi_deg ) );
set( h.EDITlightelev,'string',num2str( S.light(S.current_light).elev_deg ) );

%3D light mode toggle status
set( h.TOGGLElight,'value',S.light_toggle );

%Show vectors indicating light sources
set( h.CHECKlightarrow,'value',S.light_arrow );

%End of code